@@ -13,15 +13,12 @@ use Time::HiRes  qw(gettimeofday);
 use POSIX        qw(strftime);
 use Scalar::Util qw(looks_like_number);
 use List::Util   qw(first);
-use Cwd          qw(cwd);
-use Sys::Hostname;
 use Convert::Pheno::SQLite;
-use Convert::Pheno::Default qw(get_defaults);
+binmode STDOUT, ':encoding(utf-8)';
 use Exporter 'import';
 our @EXPORT =
-  qw(map_ontology_term dotify_and_coerce_number iso8601_time _map2iso8601 map_reference_range map_reference_range_csv map_age_range map2redcap_dict map2ohdsi convert2boolean find_age randStr map_operator_concept_id map_info_field map_omop_visit_occurrence dot_date2iso validate_format get_metaData get_info);
+  qw(map_ethnicity map_ontology dotify_and_coerce_number iso8601_time _map2iso8601 map_reference_range map_age_range map2redcap_dict map2ohdsi convert2boolean find_age randStr map_operator_concept_id map_info_field map_omop_visit_occurrence dot_date2iso remap_mapping_hash);
 
-my $DEFAULT = get_defaults();
 use constant DEVEL_MODE => 0;
 
 # Global hash
@@ -33,7 +30,16 @@ my %seen = ();
 #############################
 #############################
 
-sub map_ontology_term {
+sub map_ethnicity {
+
+    my $str       = shift;
+    my %ethnicity = ( map { $_ => 'NCIT:C41261' } ( 'caucasian', 'white' ) );
+
+# 1, Caucasian | 2, Hispanic | 3, Asian | 4, African/African-American | 5, Indigenous American | 6, Mixed | 9, Other";
+    return { id => $ethnicity{ lc($str) }, label => $str };
+}
+
+sub map_ontology {
 
     # Most of the execution time goes to this subroutine
     # We will adopt two estragies to gain speed:
@@ -42,55 +48,43 @@ sub map_ontology_term {
 
     #return { id => 'dummy', label => 'dummy' } # test speed
 
-    # We will return quickly when nothing has to be done
-    my $query = $_[0]->{query};
-
-    # Skipping numbers
-    return $DEFAULT->{ontology_term} if looks_like_number($query);
-
-    # If the ontology term is an object we assume it comes
-    # from "terminology" property in the mapping file
-    return $query if ref $query eq 'HASH';
-
     # Checking for existance in %seen
-    say "Skipping searching for <$query> as it already exists"
-      if DEVEL_MODE && exists $seen{$query};
+    my $tmp_query = $_[0]->{query};
+    say "Skipping searching for <$tmp_query> as it already exists"
+      if DEVEL_MODE && exists $seen{$tmp_query};
 
     # return if terms has already been searched and exists
     # Not a big fan of global stuff...
     #  ¯\_(ツ)_/¯
     # Premature return
-    return $seen{$query} if exists $seen{$query};    # global
+    return $seen{$tmp_query} if exists $seen{$tmp_query};    # global
+
+    say "searching for <$tmp_query>" if DEVEL_MODE;
 
     # return something if we know 'a priori' that the query won't exist
-    #return { id => 'NCIT:NA000', label => $query } if $query =~ m/xx/;
+    #return { id => 'NCIT:NA000', label => $tmp_query } if $tmp_query =~ m/xx/;
 
     # Ok, now it's time to start the subroutine
     my $arg                       = shift;
     my $column                    = $arg->{column};
     my $ontology                  = $arg->{ontology};
     my $self                      = $arg->{self};
-    my $databases                 = $self->{databases};
     my $search                    = $self->{search};
     my $print_hidden_labels       = $self->{print_hidden_labels};
     my $text_similarity_method    = $self->{text_similarity_method};
     my $min_text_similarity_score = $self->{min_text_similarity_score};
 
-    # DEVEL_MODE
-    say "searching for query <$query> in ontology <$ontology>" if DEVEL_MODE;
-
     # Die if user wants OHDSI w/o flag -ohdsi-db
     die
-"Could not find the concept_id:<$query> in the provided <CONCEPT> table.\nPlease use the flag <--ohdsi-db> to enable searching at Athena-OHDSI database\n"
+"Could not find the concept_id:<$tmp_query> in the provided <CONCEPT> table.\nPlease use the flag <--ohdsi-db> to enable searching at Athena-OHDSI database\n"
       if ( $ontology eq 'ohdsi' && !$self->{ohdsi_db} );
 
     # Perform query
-    my ( $id, $label ) = get_ontology_terms(
+    my ( $id, $label ) = get_ontology(
         {
             sth_column_ref            => $self->{sth}{$ontology}{$column},
-            query                     => $query,
+            query                     => $tmp_query,
             ontology                  => $ontology,
-            databases                 => $databases,
             column                    => $column,
             search                    => $search,
             text_similarity_method    => $text_similarity_method,
@@ -99,30 +93,26 @@ sub map_ontology_term {
     );
 
     # Add result to global %seen
-    $seen{$query} = { id => $id, label => $label };    # global
+    $seen{$tmp_query} = { id => $id, label => $label };    # global
 
 # id and label come from <db> _label is the original string (can change on partial matches)
     return $print_hidden_labels
-      ? { id => $id, label => $label, _label => $query }
+      ? { id => $id, label => $label, _label => $tmp_query }
       : { id => $id, label => $label };
 }
 
 sub dotify_and_coerce_number {
 
     my $val = shift;
-
-    # Premature return
-    return undef unless ( defined $val && $val ne '' );
-
-    # looks_like_number does not work with commas so we must tr first
     ( my $tr_val = $val ) =~ tr/,/./;
 
-    #print "#$val#$tr_val#\n";
-
-    # coercing to number $tr_val
-    return looks_like_number($tr_val)
-      ? 0 + $tr_val
-      : $val;
+    # looks_like_number does not work with commas so we must tr first
+    #say "$val === ",  looks_like_number($val);
+    # coercing to number $tr_val and avoiding value = ""
+    return
+        looks_like_number($tr_val) ? 0 + $tr_val
+      : $val eq ''                 ? undef
+      :                              $val;
 }
 
 sub iso8601_time {
@@ -163,12 +153,6 @@ sub map_reference_range {
     return $hashref;
 }
 
-sub map_reference_range_csv {
-    my ( $unit, $range ) = @_;
-    $range->{unit} = $unit;
-    return $range;
-}
-
 sub map_age_range {
 
     my $str = shift;
@@ -206,8 +190,6 @@ sub map2redcap_dict {
     #     _labels
     #  labels = 0
     #    'Field Note'
-
-    # NB: Some numeric fields will get stringified at $participant->{$field}
     return $labels
       ? $redcap_dict->{$field}{_labels}{ $participant->{$field} }
       : $redcap_dict->{$field}{'Field Note'};
@@ -216,8 +198,8 @@ sub map2redcap_dict {
 sub map2ohdsi {
 
     my $arg = shift;
-    my ( $ohdsi_dict, $concept_id, $self ) =
-      ( $arg->{ohdsi_dict}, $arg->{concept_id}, $arg->{self} );
+    my ( $ohdsi_dic, $concept_id, $self ) =
+      ( $arg->{ohdsi_dic}, $arg->{concept_id}, $arg->{self} );
 
     #######################
     # OPTION A: <CONCEPT> #
@@ -227,10 +209,10 @@ sub map2ohdsi {
     # .. we are already searching in a hash
     # NB2: $concept_id is stringified by hash
     my ( $data, $id, $label, $vocabulary ) = ( (undef) x 4 );
-    if ( exists $ohdsi_dict->{$concept_id} ) {
-        $id         = $ohdsi_dict->{$concept_id}{concept_code};
-        $label      = $ohdsi_dict->{$concept_id}{concept_name};
-        $vocabulary = $ohdsi_dict->{$concept_id}{vocabulary_id};
+    if ( exists $ohdsi_dic->{$concept_id} ) {
+        $id         = $ohdsi_dic->{$concept_id}{concept_code};
+        $label      = $ohdsi_dic->{$concept_id}{concept_name};
+        $vocabulary = $ohdsi_dic->{$concept_id}{vocabulary_id};
         $data       = { id => qq($vocabulary:$id), label => $label };
     }
 
@@ -239,7 +221,7 @@ sub map2ohdsi {
     ######################
 
     else {
-        $data = map_ontology_term(
+        $data = map_ontology(
             {
                 query    => $concept_id,
                 column   => 'concept_id',
@@ -354,27 +336,27 @@ sub map_omop_visit_occurrence {
 
     my $arg                 = shift;
     my $self                = $arg->{self};
-    my $ohdsi_dict          = $arg->{ohdsi_dict};
+    my $ohdsi_dic           = $arg->{ohdsi_dic};
     my $person_id           = $arg->{person_id};
     my $visit_occurrence_id = $arg->{visit_occurrence_id};
     my $visit_occurrence    = $self->{visit_occurrence};
 
     # Premature return
-    return undef if $visit_occurrence_id eq '\\N';    # perlcritic Severity: 5
+    return undef if $visit_occurrence_id eq '\\N';
 
 # *** IMPORTANT ***
 # EUNOMIA instance has mismatches between the person_id -- visit_occurrence_id
 # For instance, person_id = 1 has only visit_occurrence_id = 85, but on tables it has:
 # 82, 84, 42, 54, 41, 25, 76 and 81
 
-    # warn if we don't have $visit_occurrence_id in VISIT_OCCURRENCE
+    # warn if we don't have $visit_occurrence_id in VISIT_OCURRENCE
     unless ( exists $visit_occurrence->{$visit_occurrence_id} ) {
         warn
 "Sorry, but <visit_occurrence_id:$visit_occurrence_id> does not exist for <person_id:$person_id>\n"
           if DEVEL_MODE;
 
         # Premature return
-        return undef;    # perlcritic Severity: 5
+        return undef;
     }
 
     # Getting pointer to the hash element
@@ -382,7 +364,7 @@ sub map_omop_visit_occurrence {
 
     my $concept = map2ohdsi(
         {
-            ohdsi_dict => $ohdsi_dict,
+            ohdsi_dic  => $ohdsi_dic,
             concept_id => $hashref->{visit_concept_id},
             self       => $self
 
@@ -400,7 +382,7 @@ sub map_omop_visit_occurrence {
       ? $ad_hoc_44818517
       : map2ohdsi(
         {
-            ohdsi_dict => $ohdsi_dict,
+            ohdsi_dic  => $ohdsi_dic,
             concept_id => $hashref->{visit_type_concept_id},
             self       => $self
 
@@ -408,7 +390,7 @@ sub map_omop_visit_occurrence {
       );
     my $start_date = _map2iso8601( $hashref->{visit_start_date} );
     my $end_date   = _map2iso8601( $hashref->{visit_end_date} );
-    my $info       = { VISIT_OCCURRENCE => { OMOP_columns => $hashref } };
+    my $info       = { VISIT_OCCURENCE => { OMOP_columns => $hashref } };
 
     return {
         _info         => $info,
@@ -443,135 +425,22 @@ sub is_multidimensional {
     return ref shift ? 1 : 0;
 }
 
-sub validate_format {
+sub remap_mapping_hash {
 
-    my ( $data, $format ) = @_;
-
-    my $result;
-
-    # PXF
-    if ( $format eq 'pxf' ) {
-        $result = exists $data->{subject} ? 1 : 0;
-
-        # BFF
-    }
-    else {
-        $result = !exists $data->{subject} ? 1 : 0;
-    }
-    return $result;
-}
-
-sub get_info {
-
-    my $self = shift;
-
-    # Detecting the number of logical CPUs across different OSes
-    my $os = $^O;
-    chomp(
-        my $ncpuhost =
-          lc($os) eq 'darwin' || lc($os) eq 'freebsd' ? qx{sysctl -n hw.ncpu}
-        : $os eq 'MSWin32' ? qx{wmic cpu get NumberOfLogicalProcessors}
-        :                    qx{/usr/bin/nproc} // 1
-    );
-
-    # For the Windows command, the result will also contain the string
-    # "NumberOfLogicalProcessors" which is the header of the output.
-    # So we need to extract the actual number from it:
-    if ( $os eq 'MSWin32' ) {
-        ($ncpuhost) = $ncpuhost =~ /(\d+)/;
-    }
-    $ncpuhost = 0 + $ncpuhost;    # coercing it to be a number
-
-    return {
-        user => $ENV{'LOGNAME'}
-          || $ENV{'USER'}
-          || $ENV{'USERNAME'}
-          || 'dummy-user',
-        username => $self->{username},
-        ncpuhost => $ncpuhost,
-        cwd      => cwd,
-        id       => $self->{id},
-        hostname => hostname,
-        version  => $::VERSION
-    };
-}
-
-sub get_metaData {
-
-    my $self = shift;
-
-    # Setting a few variables
-    my $username = $self->{username};
-
-    # Setting resources
-    my $resources = [
-        {
-            id   => 'icd10',
-            name =>
-'International Statistical Classification of Diseases and Related Health Problems 10th Revision',
-            url             => 'https://icd.who.int/browse10/2019/en#',
-            version         => '2019',
-            namespacePrefix => 'ICD10',
-            iriPrefix       => 'https://icd.who.int/browse10/2019/en#/'
-        },
-        {
-            id              => 'ncit',
-            name            => 'NCI Thesaurus',
-            url             => 'http://purl.obolibrary.org/obo/ncit.owl',
-            version         => '22.03d',
-            namespacePrefix => 'NCIT',
-            iriPrefix       => 'http://purl.obolibrary.org/obo/NCIT_'
-        },
-        {
-            id              => 'athena-ohdsi',
-            name            => 'Athena-OHDSI',
-            url             => 'https://athena.ohdsi.org',
-            version         => 'v5.3.1',
-            namespacePrefix => 'OHDSI',
-            iriPrefix       => 'http://www.fakeurl.com/OHDSI_'
-        },
-        {
-            id              => 'hp',
-            name            => 'Human Phenotype Ontology',
-            url             => 'http://purl.obolibrary.org/obo/hp.owl',
-            version         => '2023-04-05',
-            namespacePrefix => 'HP',
-            iriPrefix       => 'http://purl.obolibrary.org/obo/HP_'
-        },
-        {
-            id              => 'omim',
-            name            => 'Online Mendelian Inheritance in Man',
-            url             => 'https://www.omim.org',
-            version         => '2023-05-22',
-            namespacePrefix => 'OMIM',
-            iriPrefix       => 'http://omim.org/entry/'
-        },
-        {
-            id   => 'cdisc-terminology',
-            name => 'CDISC Terminology',
-            url  =>
-'https://www.cdisc.org/standards/terminology/controlled-terminology',
-            version         => '2023-01-24',
-            namespacePrefix => 'CDISC',
-            iriPrefix       => 'http://www.fakeurl.com/CDISC_'
-        }
-    ];
-    return {
-        created                  => iso8601_time(),
-        createdBy                => $username,
-        submittedBy              => $username,
-        phenopacketSchemaVersion => '2.0',
-        resources                => $resources,
-        externalReferences       => [
-            {
-                id        => 'PMID: 26262116',
-                reference =>
-                  'https://www.ncbi.nlm.nih.gov/pmc/articles/PMC4815923',
-                description =>
-'Observational Health Data Sciences and Informatics (OHDSI): Opportunities for Observational Researchers'
-            }
-        ]
-    };
+    my ( $mapping_file, $term ) = @_;
+    my %hash_out = map {
+            $_, exists $mapping_file->{$term}{$_}
+          ? $mapping_file->{$term}{$_}
+          : undef
+    } (qw/fields dict map radio/);
+    $hash_out{ontology} =
+      exists $mapping_file->{$term}{ontology}
+      ? $mapping_file->{$term}{ontology}
+      : $mapping_file->{project}{ontology};
+    $hash_out{routesOfAdministration} =
+      $mapping_file->{$term}{routesOfAdministration}
+      if $term eq 'treatments';
+    return \%hash_out;
 }
 
 1;
